/*
// File:		ImapGUIDlg.cpp
//
// Version:		mail.it library 1.0
//
// Copyright:	(C) 2007 Dacons LLP.
*/

#include PCH_FILE_NAME
#include "ImapGUI.hpp"
#include "ImapGUIDlg.hpp"
#include "InputDlg.hpp"
#include "MessageDlg.hpp"
#include "SendMessageDlg.hpp"
#include <EmaiMessage.hpp>
#include "ImapFolder.hpp"
#include "ImapMessageData.hpp"
#include "MessageComposer.hpp"
#include "MessagePartEnumerator.hpp"
#include "ErrorsDescriptions.hpp"

#include <shlwapi.h>


#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CImapGUIDlg::CImapGUIDlg(CWnd* pParent /*=NULL*/)
: CDialog(CImapGUIDlg::IDD, pParent),
	mMailServer(_T("your.mailserver.net")),
	mAccount(_T("your.account")),
	mPassword(_T("password")),
	mTimerID(0),
	mImapSession(NULL, false),
	mStatusPerc(_T("")),
	mPort(EmaiDefaultImapPort),
	mUseSsl(FALSE),
	mLastRetriveMsgIndex(-1),
	mLicenseKey(_T("Put your license key here."))
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

CImapGUIDlg::~CImapGUIDlg()
{
}

void CImapGUIDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_EDIT_SERVER, mMailServer);
	DDX_Text(pDX, IDC_EDIT_ACCOUNT, mAccount);
	DDX_Text(pDX, IDC_EDIT_PASSWORD, mPassword);
	DDX_Text(pDX, IDC_STATUS_TEXT, mStatusText);
	DDX_Text(pDX, IDC_PROGRESS_PERCENT, mStatusPerc);
	DDX_Text(pDX, IDC_EDIT_PORT, mPort);
	DDX_Check(pDX, IDC_CHECK_SSL, mUseSsl);
	DDX_Control(pDX, IDC_COMBO_AUTH_TYPE, mAuthType);
	DDX_Control(pDX, IDC_LIST_MAILBOXES, mMailboxesList);
	DDX_Control(pDX, IDC_LIST_MESSAGES, mMessagesList);
	DDX_Control(pDX, IDC_BUTTON_ANOTHER, mAnotherServer);
	DDX_Control(pDX, IDC_EDIT_SERVER, mMailServerEdit);
	DDX_Control(pDX, IDC_EDIT_ACCOUNT, mAccountEdit);
	DDX_Control(pDX, IDC_EDIT_PASSWORD, mPasswordEdit);
	DDX_Control(pDX, IDC_EDIT_PORT, mPortEdit);
	DDX_Control(pDX, IDC_CHECK_SSL, mUseSslButton);
	DDX_Control(pDX, IDC_BUTTON_GETMAILBOX, mGetMailboxes);
	DDX_Text(pDX, IDC_EDIT_LICKEY, mLicenseKey);
	DDX_Control(pDX, IDC_EDIT_LICKEY, mLicenseKeyEdit);
}

BEGIN_MESSAGE_MAP(CImapGUIDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_TIMER()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_BUTTON_GETMAILBOX, OnBnClickedButtonGetmailboxes)
	ON_BN_CLICKED(IDC_BUTTON_ADDFOLDER, OnBnClickedButtonAddMailbox)
	ON_BN_CLICKED(IDC_BUTTON_DELFOLDER, OnBnClickedButtonDeleleteMailbox)
	ON_BN_CLICKED(IDC_BUTTON_DELFILE, OnBnClickedButtonDeleteMessage)
	ON_BN_CLICKED(IDC_BUTTON_ADDFILE, OnBnClickedButtonAddMessage)
	ON_NOTIFY(LVN_ITEMACTIVATE, IDC_LIST_MESSAGES, OnLvnItemActivateListMessages)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_LIST_MAILBOXES, OnCustomDrawMailboxes)
	ON_NOTIFY(LVN_GETDISPINFO, IDC_LIST_MAILBOXES, OnGetMailboxesDispInfoList)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_LIST_MESSAGES, OnCustomDrawMessages)
	ON_NOTIFY(LVN_GETDISPINFO, IDC_LIST_MESSAGES, OnGetMessagesDispInfoList)
	ON_BN_CLICKED(IDC_BUTTON_EXPUNGE, OnBnClickedButtonExpunge)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_LIST_MAILBOXES, OnItemchangedListMailboxes)
	ON_BN_CLICKED(IDC_BUTTON_ANOTHER, OnBnClickedButtonAnother)
	ON_NOTIFY(LVN_ENDLABELEDIT, IDC_LIST_MAILBOXES, OnLvnEndlabeleditListMailboxes)
	ON_BN_CLICKED(IDC_BUTTON_RENAME, OnBnClickedButtonRename)
	ON_BN_CLICKED(IDC_BUTTON_UNDELETE, OnBnClickedButtonUndelete)
	ON_NOTIFY(LVN_ODCACHEHINT, IDC_LIST_MESSAGES, OnLvnOdcachehintListMessages)
	ON_EN_KILLFOCUS(IDC_EDIT_LICKEY, OnEnKillfocusEditLickey)
	ON_EN_SETFOCUS(IDC_EDIT_LICKEY, OnEnSetfocusEditLickey)
END_MESSAGE_MAP()

BOOL CImapGUIDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	mAuthType.SetCurSel(0);

	mMailboxesList.InsertColumn(0, _T("Mailboxes"), LVCFMT_CENTER, -1, 0);
	mMailboxesList.SetColumnWidth(0, LVSCW_AUTOSIZE_USEHEADER);
	mMailboxesList.SetExtendedStyle(LVS_EX_FULLROWSELECT);

	mMessagesList.InsertColumn(0, _T("Messages"), LVCFMT_CENTER, -1, 0);
	mMessagesList.SetColumnWidth(0, LVSCW_AUTOSIZE_USEHEADER);
	mMessagesList.SetExtendedStyle(LVS_EX_FULLROWSELECT);

	// Set timer in 60 seconds
	SetTimer(mTimerID, 60*1000, NULL);

	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CImapGUIDlg::OnDestroy()
{
	// Stop timer
	KillTimer(mTimerID);
	// End session and disconnect from server
	if (mImapSession != NULL)
		mImapSession.Disconnect();
}

void CImapGUIDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting
	
		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CImapGUIDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CImapGUIDlg::OnTimer(UINT nIDEvent)
{
	try
	{
		if (mImapSession != NULL)
		{
			mImapSession.Noop(); //Update message list
			ImapFolder* folder = GetActiveMailbox();
			if ((mMailboxesList.GetSelectionMark() >= 0) && (folder != NULL))
			{
				mMailboxesList.SetItemCount((int)mImapResponseHandler.GetFoldersCount());
				mMessagesList.SetItemCount(folder->GetExists());
			}
		}
	}
	catch (Emai::Exception& exception)
	{
		//reset IMAP session object
		ResetConnection();
		FormatError(exception);
	}
	CDialog::OnTimer(nIDEvent);
}

ImapFolder* CImapGUIDlg::GetActiveMailbox()
{
	int selectedRow = mMailboxesList.GetSelectionMark();
	if ((mImapResponseHandler.GetFoldersCount() > 0) && (selectedRow != -1))
	{
		Emai::UniString name = mImapResponseHandler.GetFolderAtIndex(selectedRow)->GetName();
		if (name.length() > 0)
		{
			ImapFolder* folder = mImapResponseHandler.FindFolderByName(name.c_str());
			return folder;
		}
	}
	return NULL;
}

bool CImapGUIDlg::ConnectSession()
{
	try
	{
		UpdateData();
		if (mImapSession == NULL)
		{
			EmaiOptions options = EmaiNullOptions;
			if (mUseSsl) 
				options = EmaiImapUseSSLOption;
			mImapSession = Emai::ImapSession(&mImapResponseHandler,
				mStatusCallback, 60, options);
			HWND hwnd = *GetDlgItem(IDC_PROGRESS_PERCENT);

			// Set handle of progress percent static text control as user data to change its in statusCallback
			int perc = 0;
			mImapSession.SetUserData(ImapCurrentPercent, &perc, sizeof(EmaiUint32));
			mImapSession.SetUserData(ImapPercentText, &hwnd, sizeof(HWND));
			hwnd = *GetDlgItem(IDC_STATUS_TEXT);
			mImapSession.SetUserData(ImapStatusText, &hwnd, sizeof(HWND));

			// Connect to server
			mImapSession.Connect(mMailServer, mPort);

			// Authenticate with user name and password
			EmaiUint32 authTypes[] = {
					EmaiIMAPAuthAutomatic,
					EmaiIMAPAuthLogin,
					EmaiIMAPAuthPlain,
					EmaiIMAPAuthCramMD5,
					EmaiIMAPAuthNTLM,
					EmaiIMAPAuthLoginCommand
			};
			mImapSession.Authenticate(mAccount, mPassword, authTypes[mAuthType.GetCurSel()]);
		}
	}
	catch (Emai::Exception& exception)
	{
		//reset IMAP session object
		ResetConnection();
		FormatError(exception);
		return false;
	}
	return true;
}

bool CImapGUIDlg::FetchData(EmaiFetchMessageAttribute inFetchAttr[], 
							EmaiUint32 attributeCount, 
							Emai::ImapSequenceSet& msgSequence)
{
	try
	{
		EmaiFetchMessageData fetchData = {EmaiStructureVersionFirst};
		fetchData.selector = EmaiDataFetchAttr;
		fetchData.param.attr.attributeCount = attributeCount;
		fetchData.param.attr.attributes = inFetchAttr;

		// Fetch data from server for message with number inMessageNumber
		mImapSession.Fetch(msgSequence, &fetchData);
		mStatusText = _T("");
		UpdateData(FALSE);
		return true;
	}
	catch (Emai::Exception& exception)
	{
		//reset IMAP session object
		mStatusText = _T("");
		UpdateData(FALSE);
		FormatError(exception);
		return false;
	}
}

void CImapGUIDlg::OnBnClickedButtonGetmailboxes()
{
	UpdateData();
	if (EmaiRegisterLibrary(mLicenseKey) == EmaiErrNoErr)
	{
		try
		{
			mMailboxesList.SetItemCount(0);
			mMessagesList.SetItemCount(0);

			if (mImapSession != NULL)
				mImapSession = Emai::ImapSession(NULL, true);

			// Connect to server if is not connected
			if (!ConnectSession())
				return;

			mImapResponseHandler.ClearFolderList();

			/*Create the list of the folders*/
			mImapSession.ListMailboxes(_T(""), _T("*"));

			mMailboxesList.SetItemCount((int)mImapResponseHandler.GetFoldersCount());
			LoadMessages(-1);

			mStatusText = _T("");
			mAnotherServer.EnableWindow(TRUE);
			mMailServerEdit.EnableWindow(FALSE);
			mAccountEdit.EnableWindow(FALSE);
			mPasswordEdit.EnableWindow(FALSE);
			mPortEdit.EnableWindow(FALSE);
			mUseSslButton.EnableWindow(FALSE);
			mGetMailboxes.EnableWindow(FALSE);
			mAuthType.EnableWindow(FALSE);
			mLicenseKeyEdit.EnableWindow(FALSE);
			UpdateData(FALSE);
		}
		catch (Emai::Exception& exception)
		{
			//reset IMAP session object
			mImapSession = Emai::ImapSession(NULL, true);
			mStatusText = _T("");
			UpdateData(FALSE);
			FormatError(exception);
		}
	}
	else
		AfxMessageBox(_T("Licence key is invalid. Restart application and enter valid key."));
}

void CImapGUIDlg::OnBnClickedButtonAddMailbox()
{
	if (mImapSession != NULL)
	{
		try
		{
			CInputDlg dlg(_T("Create mailbox"));
			if (dlg.DoModal() == IDOK)
			{
				ImapFolder* folder = mImapResponseHandler.FindFolderByName(dlg.mName);

				if (folder == NULL)
				{
					/*Create the folder*/
					mImapSession.CreateMailbox(dlg.mName);
					/*Add new mailbox to mailboxes list*/
					mImapSession.ListMailboxes(_T(""), dlg.mName);
					mMailboxesList.SetItemCount((int)mImapResponseHandler.GetFoldersCount());
				}
				else
				{
					AfxMessageBox(_T("Folder ") + dlg.mName + _T("already exist."));
				}
			}
			mStatusText = _T("");
			UpdateData(FALSE);
		}
		catch (Emai::Exception& exception)
		{
			//reset IMAP session object
			mStatusText = _T("");
			UpdateData(FALSE);
			FormatError(exception);
		}
	}
}

void CImapGUIDlg::OnBnClickedButtonDeleleteMailbox()
{
	if ((mImapSession != NULL) && (mMailboxesList.GetSelectedCount() > 0))
	{
		if (AfxMessageBox(_T("Do you really want to delete the selected folder?"),
			MB_ICONQUESTION | MB_YESNO) == IDYES)
		{
			try
			{
				ImapFolder* folder = GetActiveMailbox();
				ASSERT(folder != NULL);
				/*Delete the folder*/
				mImapSession.DeleteMailbox(folder->GetName().c_str());
				mImapResponseHandler.DeleteFolderFromFolderList(folder->GetName().c_str());
				mMailboxesList.SetItemCount((int)mImapResponseHandler.GetFoldersCount());
				LoadMessages(-1);
				mStatusText = _T("");
				UpdateData(FALSE);
			}
			catch (Emai::Exception& exception)
			{
				//reset IMAP session object
				mStatusText = _T("");
				UpdateData(FALSE);
				FormatError(exception);
			}
		}
	}
}

void CImapGUIDlg::OnBnClickedButtonDeleteMessage()
{
	if ((mImapSession != NULL) && (mMessagesList.GetSelectedCount() > 0))
	{
		if (AfxMessageBox(_T("Do you really want to mark selected message(s) as deleted?"),
			MB_ICONQUESTION | MB_YESNO) == IDYES)
		{
			try
			{
				ImapFolder* folder = GetActiveMailbox();
				ASSERT(folder != NULL);
				EmaiStoreMessageData storeData = {EmaiStructureVersionFirst};
				storeData.selector = EmaiDataStoreAttr;
				storeData.param.attr.options = EmaiStoreAddFlags;
				storeData.param.attr.flags.flags = EmaiDeletedFlag;
				storeData.param.attr.flags.customFlags = NULL;

				Emai::ImapSequenceSet msgSequenceUid;
				POSITION pos = mMessagesList.GetFirstSelectedItemPosition();
				while (pos != NULL)
				{
					msgSequenceUid.AddNumber(folder->GetMessageDataAtIndex(mMessagesList.GetNextSelectedItem(pos) + 1)->GetUid());
				}
				if (msgSequenceUid.GetCount() > 0)
				{
					/*Select the current message*/
					if (folder->GetSelectedForDeletion() == NULL)
						folder->SetSelectedForDeletion(msgSequenceUid);
					else
						folder->GetSelectedForDeletion().AddSequenceSet(msgSequenceUid);

					mImapSession.Store(msgSequenceUid, &storeData, EmaiImapUIDOption);
					LoadMessages(mMailboxesList.GetSelectionMark());
				}
			}
			catch (Emai::Exception& exception)
			{
				//reset IMAP session object
				mStatusText = _T("");
				UpdateData(FALSE);
				FormatError(exception);
			}
		}
	}
}

CString CImapGUIDlg::FormatTextForTextField(EmaiOptions inFieldType, 
								   MessagePartEnumerator& inEnumerator) const
{
	CString str = _T("");
	Emai::ContactList contacts = inEnumerator.GetContacts(inFieldType);
	EmaiUint32 count = contacts.GetCount();
	CString name;
	CString address;
	EmaiUint32 i = 0;
	for (i = 0; i < count; ++i)
	{
		name = contacts.GetName(i);
		address = contacts.GetAddress(i);
		if ((name.GetLength() > 0) && (address.GetLength() > 0))
			str = str + name + _T(" - ") + address + _T("; ");
		else if ((name.GetLength() > 0) && (address.GetLength() == 0))
			str = str + name + _T("; ");
		else if ((name.GetLength() == 0) && (address.GetLength() > 0))
			str = str + address + _T("; ");
	}
	return str;
}

void CImapGUIDlg::FormatError(Emai::Exception& inException)
{
	AfxMessageBox(errorsDescriptions[::abs(inException.GetErrorCode())]);
}

void CImapGUIDlg::OnBnClickedButtonAddMessage()
{
	if (mImapSession != NULL)
	{
		CSendMessageDlg dlg;
		if (dlg.DoModal() == IDOK)
		{
			CString message;
			if (dlg.mTo.IsEmpty())
				message = _T("You must enter recipient address.");
			else if (dlg.mFrom.IsEmpty())
				message = _T("You must enter sender address.");

			if (!message.IsEmpty())
				AfxMessageBox(message);
			else
			{
				try
				{
					ImapFolder* folder = GetActiveMailbox();
					ASSERT(folder != NULL);
					/*Compose the message*/

					/*Create the message composer*/
					MessageComposer messageComposer;

					/*Add the message text part*/
					messageComposer.SetUnicodeText(dlg.mMessage);

					/*Add contact fields to the message*/
					Emai::ContactList contactTo;
					contactTo.AddContact(dlg.mTo, NULL);
					Emai::ContactList contactFrom;
					contactFrom.AddContact(dlg.mFrom, NULL);

					Emai::Message message = messageComposer.ComposeMessage(contactFrom, NULL, contactTo,
						NULL, NULL, dlg.mSubject);

					if (message != NULL)
					{
						/*Add the message to the folder*/
						mImapSession.AppendMessage(folder->GetName().c_str(), NULL, NULL, message);
						mStatusPerc = _T("");
						UpdateData(FALSE);
						LoadMessages(mMailboxesList.GetSelectionMark());
					}
				}
				catch (Emai::Exception& exception)
				{
					//reset IMAP session object
					mStatusText = _T("");
					mStatusPerc = _T("");
					UpdateData(FALSE);
					FormatError(exception);
				}
			}
		}
	}
}

void CImapGUIDlg::LoadMessages(int inMailbox)
{
	try
	{
		int mailbox = inMailbox;
		int index = mMailboxesList.GetSelectionMark();
		if (index != -1)
		{
			if (inMailbox == -1)
				mailbox = index;
		}
		else
			mailbox = 0;
		// Connect to server if is not connected
		if (!ConnectSession())
			return;
	
		CString folderName = mMailboxesList.GetItemText(mailbox, 0);
		ImapFolder* folder = mImapResponseHandler.FindFolderByName(folderName);
		ASSERT(folder != NULL);

		if ((folder->GetAttributes() & EmaiStandardAttibutesMask) != EmaiNoselectAttribute)
		{
			/*Choose the folder*/
			mImapSession.SelectMailbox(folderName);

			mMessagesList.SetItemCount(folder->GetExists());
			if (index == -1)
			{
				mMailboxesList.SetItemState(mailbox, LVIS_SELECTED|LVIS_FOCUSED, LVIS_SELECTED|LVIS_FOCUSED);
				mMailboxesList.SetFocus();
			}
			mMessagesList.UpdateData(FALSE);
			mStatusText = _T("");
			UpdateData(FALSE);
		}
	}
	catch (Emai::Exception& exception)
	{
		//reset IMAP session object
		mImapSession = Emai::ImapSession(NULL, true);
		mStatusText = _T("");
		UpdateData(FALSE);
		FormatError(exception);
	}
}

void CImapGUIDlg::OnLvnItemActivateListMessages(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMITEMACTIVATE pNMIA = reinterpret_cast<LPNMITEMACTIVATE>(pNMHDR);
	try
	{
		if (!ConnectSession())
			return;

		ImapFolder* folder = GetActiveMailbox();
		ASSERT(folder != NULL);
		if ((folder->GetAttributes() & EmaiStandardAttibutesMask) != EmaiNoselectAttribute)
		{
			/*Select the folder. For broken connection.*/
			mImapSession.SelectMailbox(folder->GetName().c_str());

			int index = pNMIA->iItem;
			if (index != -1)
			{
				EmaiUint32 messageNumber = index + 1;
				bool fetchResult = true;
				if (!folder->GetMessageDataAtIndex(messageNumber)->HasFullMessageData()) 
				{
					EmaiFetchMessageAttribute attr[] = {{EmaiFetchRFC822Text}};
					Emai::ImapSequenceSet msgSequence;
					msgSequence.AddNumber(messageNumber);
					fetchResult = FetchData(attr, 1, msgSequence);
				}

				if (fetchResult)
				{
					Emai::Message message = folder->GetMessageDataAtIndex(messageNumber)->GetMessage();
					mStatusPerc = _T("");
					UpdateData(FALSE);

					/*Create the message enumerator*/
					MessagePartEnumerator messagePartEnumerator(message);
					messagePartEnumerator.Enumerate();

					CMessageDlg dlg;

					// Fill dialog fields with data
					dlg.mFrom = FormatTextForTextField(EmaiFrom, messagePartEnumerator);
					dlg.mTo = FormatTextForTextField(EmaiTo, messagePartEnumerator);
					dlg.mCC = FormatTextForTextField(EmaiCC, messagePartEnumerator);
					dlg.mSubject = messagePartEnumerator.GetSubject().c_str();

					// Fill message field with message text
					Emai::UniString msg;
					messagePartEnumerator.GetUnicodeText(msg);
					dlg.mMessage = CString(msg.c_str());

					// Fill attachments list with attachments
					EmaiUint32 attachmentsCount = messagePartEnumerator.GetAttachmentsCount();
					EmaiUint32 length = 0;
					TCHAR buffer[256];
					CString attachmentInfo;
					if (attachmentsCount > 0)
					{
						Emai::UniString fileName;
						int count = 0;
						for (EmaiUint32 i = 0; i < attachmentsCount; i++)
						{
							messagePartEnumerator.GetAttachmentAtIndex(i, fileName, NULL, length);
							::StrFormatByteSize(length, buffer, sizeof(buffer));
							attachmentInfo.Format(_T("%s (size = %s)"), fileName.c_str(), buffer);
							dlg.AddString(attachmentInfo);
						}
					}

					// Fill attachments list with inline attachments
					attachmentsCount = messagePartEnumerator.GetInlinesCount();
					Emai::UniString conId;
					if (attachmentsCount > 0)
					{
						Emai::UniString fileName;
						int count = 0;
						for (EmaiUint32 i = 0; i < attachmentsCount; i++)
						{
							messagePartEnumerator.GetInlineAtIndex(i, fileName, conId, NULL, length);
							::StrFormatByteSize(length, buffer, sizeof(buffer));
							attachmentInfo.Format(_T("%s (size = %s content ID = %s)"),fileName.c_str(), buffer, conId.c_str());
							dlg.AddString(attachmentInfo);
						}
					}
					dlg.DoModal();
				}
			}
			else
			{
					CString msg;
					msg.Format(_T("There is no folder %s in mail account."), folder->GetName().c_str());
					AfxMessageBox(msg);
			}
		}
	}
	catch (Emai::Exception& exception)
	{
		mStatusText = _T("");
		mStatusPerc = _T("");
		UpdateData(FALSE);
		FormatError(exception);
	}
	*pResult = 0;
}

void CImapGUIDlg::OnCustomDrawMailboxes(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LPNMLVCUSTOMDRAW lplvcd = reinterpret_cast<LPNMLVCUSTOMDRAW>(pNMHDR);

	if (lplvcd->nmcd.dwDrawStage == CDDS_PREPAINT)
	{
	    *pResult = CDRF_NOTIFYITEMDRAW;
	}
    else if (lplvcd->nmcd.dwDrawStage == CDDS_ITEMPREPAINT)
	{
	    *pResult = CDRF_NOTIFYSUBITEMDRAW;
	}
    else if (lplvcd->nmcd.dwDrawStage == (CDDS_ITEMPREPAINT | CDDS_SUBITEM))
    {
		// Load color from item data and set it as text color
		ImapFolder* folder = mImapResponseHandler.GetFolderAtIndex((EmaiUint32)lplvcd->nmcd.dwItemSpec);
		ASSERT(folder != NULL);
		if ((folder->GetAttributes() & EmaiStandardAttibutesMask) == EmaiNoselectAttribute)
			lplvcd->clrText = RGB(190, 190, 190);
		*pResult = CDRF_DODEFAULT;
	}
}


void CImapGUIDlg::OnGetMailboxesDispInfoList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_DISPINFO* pDispInfo = reinterpret_cast<LV_DISPINFO*>(pNMHDR);
	LV_ITEM* pItem= &(pDispInfo)->item;
	if (pItem->mask & LVIF_TEXT)
	{
		if(pItem->iSubItem == 0)
		{
			CString text = mImapResponseHandler.GetFolderAtIndex(pItem->iItem)->GetName().c_str();
			::lstrcpyn(pItem->pszText, text, pItem->cchTextMax);
		}
	}

	*pResult = 0;
}

void CImapGUIDlg::OnCustomDrawMessages(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LPNMLVCUSTOMDRAW lplvcd = reinterpret_cast<LPNMLVCUSTOMDRAW>(pNMHDR);

	if (lplvcd->nmcd.dwDrawStage == CDDS_PREPAINT)
	{
	    *pResult = CDRF_NOTIFYITEMDRAW;
	}
    else if (lplvcd->nmcd.dwDrawStage == CDDS_ITEMPREPAINT)
	{
	    *pResult = CDRF_NOTIFYSUBITEMDRAW;
	}
    else if (lplvcd->nmcd.dwDrawStage == (CDDS_ITEMPREPAINT | CDDS_SUBITEM))
    {
		// Load color from item data and set it as text color
		ImapFolder* folder = GetActiveMailbox();
		ASSERT(folder != NULL);
		if (folder->GetExists() > 0)
		{
			if (folder->GetMessageDataAtIndex((EmaiUint32)lplvcd->nmcd.dwItemSpec + 1) != NULL)
			{
				if (folder->GetMessageDataAtIndex((EmaiUint32)lplvcd->nmcd.dwItemSpec + 1)->GetFlags() & EmaiDeletedFlag)
				{
					if (folder->IsMarkedForDeletion((EmaiUint32)lplvcd->nmcd.dwItemSpec + 1))
						lplvcd->clrText = RGB(200, 0, 0);
					else
						lplvcd->clrText = RGB(255, 0, 0);
				}
			}
		}
		*pResult = CDRF_DODEFAULT;
	}
}

void CImapGUIDlg::OnLvnOdcachehintListMessages(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMLVCACHEHINT pCacheHint = reinterpret_cast<LPNMLVCACHEHINT>(pNMHDR);
	ImapFolder* folder = GetActiveMailbox();
	ASSERT(folder != NULL);
	if (folder->GetExists() > 0)
	{
		Emai::ImapSequenceSet msgSequence;
		EmaiFetchMessageAttribute fetchAttr[] = {{EmaiFetchFlags}, {EmaiFetchRFC822Header}, {EmaiFetchUid}};
		for (int i = pCacheHint->iFrom; i <= pCacheHint->iTo; i++)
		{
			if (folder->GetMessageDataAtIndex(i + 1) == NULL)
			{
				msgSequence.AddNumber(i + 1);
			}
		}
		if (msgSequence.GetCount() > 0)
		{
			FetchData(fetchAttr, 3, msgSequence);
		}
	}
	*pResult = 0;
}

void CImapGUIDlg::OnGetMessagesDispInfoList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_DISPINFO* pDispInfo = reinterpret_cast<LV_DISPINFO*>(pNMHDR);
	LV_ITEM* pItem= &(pDispInfo)->item;
	if (pItem->mask & LVIF_TEXT)
	{
		if(pItem->iSubItem == 0)
		{
			if (mLastRetriveMsgIndex != pItem->iItem)
			{
				mLastRetriveMsgIndex = pItem->iItem;
				ImapFolder* folder = GetActiveMailbox();
				ASSERT(folder != NULL);
				if (folder->GetExists() > 0)
				{
					if (folder->GetMessageDataAtIndex(pItem->iItem + 1) != NULL)
					{
						Emai::Message message = folder->GetMessageDataAtIndex(pItem->iItem + 1)->GetMessage();
						mLastRetriveMsgItem = message.GetField(NULL, EmaiSubject);
					}
					else
						mLastRetriveMsgItem = _T("");
				}
			}
			::lstrcpyn(pItem->pszText, mLastRetriveMsgItem.GetBuffer(mLastRetriveMsgItem.GetLength()), pItem->cchTextMax);
		}
	}
	*pResult = 0;
}
void CImapGUIDlg::OnBnClickedButtonExpunge()
{
	try
	{
		ImapFolder* folder = GetActiveMailbox();
		if (folder != NULL)
		{
			/*Expunge all messages, which have been selected*/
			if (folder->GetSelectedForDeletion() != NULL)
			{
				if (AfxMessageBox(_T("Do you really want to delete message(s) from server?"),
					MB_ICONQUESTION | MB_YESNO) == IDYES)
				{
					mImapSession.Expunge(folder->GetSelectedForDeletion());
					folder->ResetSelectedForDeletion();
					LoadMessages(mMailboxesList.GetSelectionMark());
					mStatusText = _T("");
					UpdateData(FALSE);
				}
			}
		}
	}
	catch (Emai::Exception& exception)
	{
		//reset IMAP session object
		mStatusText = _T("");
		UpdateData(FALSE);
		FormatError(exception);
	}
}

void CImapGUIDlg::OnItemchangedListMailboxes(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);
	LoadMessages(pNMLV->iItem);
	*pResult = 0;
}

void CImapGUIDlg::OnBnClickedButtonAnother()
{
	UpdateData(FALSE);
	if (mImapSession != NULL)
		mImapSession.Disconnect();
	ResetConnection();
}

void CImapGUIDlg::OnLvnEndlabeleditListMailboxes(NMHDR *pNMHDR, LRESULT *pResult)
{
	NMLVDISPINFO *pDispInfo = reinterpret_cast<NMLVDISPINFO*>(pNMHDR);
	if (pDispInfo->item.pszText != NULL)
	{
		try
		{
			ImapFolder* folder = mImapResponseHandler.GetFolderAtIndex(pDispInfo->item.iItem);
			ASSERT(folder != NULL);
			if (::wcsicmp(folder->GetName().c_str(), pDispInfo->item.pszText) != 0)
			{
				mImapSession.RenameMailbox(folder->GetName().c_str(), pDispInfo->item.pszText);
				mImapResponseHandler.RenameFoldersInFolderList(folder->GetName().c_str(), pDispInfo->item.pszText);
				LoadMessages(-1);
				mStatusText = _T("");
				UpdateData(FALSE);
			}
		}
		catch (Emai::Exception& exception)
		{
			//reset IMAP session object
			mStatusText = _T("");
			UpdateData(FALSE);
			FormatError(exception);
		}
	}
	*pResult = 0;
}

void CImapGUIDlg::OnBnClickedButtonRename()
{
	if (mImapSession != NULL)
	{
		try
		{
			CInputDlg dlg(_T("Rename mailbox"));
			if (dlg.DoModal() == IDOK)
			{
				ImapFolder* folder = GetActiveMailbox();
				ASSERT(folder != NULL);
				if (::wcsicmp(folder->GetName().c_str(), dlg.mName) != 0)
				{
					mImapSession.RenameMailbox(folder->GetName().c_str(), dlg.mName);
					mImapResponseHandler.RenameFoldersInFolderList(folder->GetName().c_str(), dlg.mName);
					LoadMessages(-1);
					mStatusText = _T("");
					UpdateData(FALSE);
				}
			}
		}
		catch (Emai::Exception& exception)
		{
			//reset IMAP session object
			mStatusText = _T("");
			UpdateData(FALSE);
			FormatError(exception);
		}
	}
}

void CImapGUIDlg::ResetConnection()
{
	mImapResponseHandler.ClearFolderList();
	mImapSession = Emai::ImapSession(NULL, true);
	mMailboxesList.SetItemCount(0);
	mMessagesList.SetItemCount(0);
	mAnotherServer.EnableWindow(FALSE);
	mMailServerEdit.EnableWindow(TRUE);
	mAccountEdit.EnableWindow(TRUE);
	mPasswordEdit.EnableWindow(TRUE);
	mPortEdit.EnableWindow(TRUE);
	mUseSslButton.EnableWindow(TRUE);
	mGetMailboxes.EnableWindow(TRUE);
	mAuthType.EnableWindow(TRUE);
	mLicenseKeyEdit.EnableWindow(TRUE);
	mMailServerEdit.SetFocus();
	mStatusText = _T("");
	UpdateData(FALSE);
}


void CImapGUIDlg::OnBnClickedButtonUndelete()
{
	try
	{
		ImapFolder* folder = GetActiveMailbox();
		ASSERT(folder != NULL);
		EmaiStoreMessageData storeData = {EmaiStructureVersionFirst};
		storeData.selector = EmaiDataStoreAttr;
		storeData.param.attr.options = EmaiStoreRemoveFlags;
		storeData.param.attr.flags.flags = EmaiDeletedFlag;
		storeData.param.attr.flags.customFlags = NULL;

		Emai::ImapSequenceSet msgSequenceUid;
		POSITION pos = mMessagesList.GetFirstSelectedItemPosition();
		EmaiUint32 item = 0;
		while (pos != NULL)
		{
			item = mMessagesList.GetNextSelectedItem(pos) + 1;
			msgSequenceUid.AddNumber(folder->GetMessageDataAtIndex(item)->GetUid());
			folder->GetMessageDataAtIndex(item)->SetFlags(folder->GetMessageDataAtIndex(item)->GetFlags() & ~EmaiDeletedFlag);
		}

		if (msgSequenceUid.GetCount() > 0)
		{
			folder->GetSelectedForDeletion().RemoveSequenceSet(msgSequenceUid);
			/*Select the current message*/
			mImapSession.Store(msgSequenceUid, &storeData, EmaiImapUIDOption);
			LoadMessages(mMailboxesList.GetSelectionMark());
		}
	}
	catch (Emai::Exception& exception)
	{
		//reset IMAP session object
		mStatusText = _T("");
		UpdateData(FALSE);
		FormatError(exception);
	}
}


void CImapGUIDlg::OnEnKillfocusEditLickey()
{
	UpdateData();
	if (mLicenseKey.IsEmpty())
	{
		mLicenseKey = _T("Put your license key here.");
		UpdateData(FALSE);
	}
}

void CImapGUIDlg::OnEnSetfocusEditLickey()
{
	UpdateData();
	if (mLicenseKey.CompareNoCase(_T("Put your license key here.")) == 0)
	{
		mLicenseKey = _T("");
		UpdateData(FALSE);
	}
}
