/*
// File:		Pop3Callback.mm
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2007 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/
#include PCH_FILE_NAME
#include "Pop3Callback.h"
#include <iostream>
#include <assert.h>

EmaiError
Pop3Callback::OnOperation(
		Emai::Pop3Session& inSession,
		EmaiUint32 inOperationCode
	)
{
	EmaiUint32 statSize = sizeof(NSTextField*);
	EmaiUint32 numSize = sizeof(EmaiUint32);
	NSTextField* status = NULL;
	NSString* text = NULL;
	inSession.GetUserData(Pop3StatusText, &status, statSize);
	NSString* statusTexts[] = {
		@"Connecting to server...",
		@"Successfully connected to server.",
		@"Authenticating on server...",
		@"Successfully authenticated on server.",
		@"Getting message list from server...",
		@"Successfully got messages from server.",
		@"Getting message UIDLs...",
		@"Successfully got messages UIDLs.",
		@"Retrieving message %d of %d.",
		@"Successfully retrieved messages.",
		@"Deletting message %d of %d.",
		@"Message successfully deleted.",
		@"Sending NOOP...",
		@"NOOP successfully sent.",
		@"Closing connection.",
		@"Connection successfully closed."
	};
	
	switch (inOperationCode)
	{
		case EmaiPop3DeletingMessage:
		case EmaiPop3RetrievingMessage:
		{
			EmaiUint32 curMessage = 0;
			EmaiUint32 messageNumber = 0;
			inSession.GetUserData(Pop3CurrentMessage, &curMessage, numSize);
			inSession.GetUserData(Pop3MessageNumber, &messageNumber, numSize);
			text = [NSString stringWithFormat:statusTexts[inOperationCode],
				curMessage + 1, messageNumber];
			break;
		}
		default:
			assert((inOperationCode >= EmaiPop3ConnectingToServer) &&
							(inOperationCode <= EmaiPop3ConnectionClosed));
			text = statusTexts[inOperationCode];
			break;
	}
	
	[status setStringValue:text];
	[status displayIfNeeded];
	
	return EmaiErrNoErr;
}

EmaiError
Pop3Callback::OnDataReceived(
		Emai::Pop3Session& inSession,
		EmaiUint32 inReceivedSize,
		EmaiUint32 inTotalSize
	)
{
	EmaiUint32 perSize = sizeof(NSTextField*);
	int percent = 0;
	int curPercent = (100 * inReceivedSize) / inTotalSize;
	EmaiUint32 numSize = sizeof(EmaiUint32);
	inSession.GetUserData(Pop3CurrentPercent, &percent, numSize);
	if (percent != curPercent)
	{
		NSTextField* status = NULL;
		inSession.GetUserData(Pop3PercentText, &status, perSize);
		[status setStringValue:[NSString stringWithFormat: @"%d %%", curPercent]];
		[status displayIfNeeded];
		inSession.SetUserData(Pop3CurrentPercent, &curPercent, numSize);
	}
	return EmaiErrNoErr;
}

EmaiError
Pop3Callback::OnServerError(
		Emai::Pop3Session& inSession,
		EmaiConstCharPtr inServerReply,
		EmaiUint32 /*inReplyCode*/
	)
{
	EmaiUint32 statSize = sizeof(NSTextField*);
	NSTextField* status = NULL;
	NSString* text = [NSString stringWithFormat:@"The server error encountered was: %s", inServerReply];
	inSession.GetUserData(Pop3StatusText, &status, statSize);
	[status setStringValue:text];
	[status displayIfNeeded];
	return EmaiErrNoErr;
}
