/*
// File:		SendMessages.cpp
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2006 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#include PCH_FILE_NAME
#include "SmtpCallback.hpp"
#include <EmaiSmtp.hpp>
#include <EmaiMessage.hpp>
#include <EmaiMessage.h>
#include <MessageComposer.hpp>
#include <iostream>

/*SMTP account settings, replace them with real account settings*/
EmaiUint32 sTimeout							= 60;
EmaiUniChar sMailServer[]					= {'m', 'a', 'i', 'l', '.', 'm', 'y', 'd', 'o', 'm', 'a', 'i', 'n', '.', 'c', 'o', 'm', '\0'};
EmaiUniChar sMailServerAccount[]			= {'m', 'y', 'a', 'c', 'c', 'o', 'u', 'n', 't', '\0'};
EmaiUniChar sMailServerPassword[]			= {'m', 'y', 'p', 'a', 's', 's', 'w', 'o', 'r', 'd', '\0'};
EmaiOptions sAuthType						= EmaiSMTPAuthAutomatic;

EmaiUniChar sContactToName[]				= {'T', 'o', 'm', '\0'};
//EmaiUniChar sContactToAddress[]			= {'t', 'o', 'm', '@', 'm', 'y', 'd', 'o', 'm', 'a', 'i', 'n', '.', 'c', 'o', 'm', '\0'};
EmaiUniChar sContactToAddress[]				= {'s', 'p', 'a', 'm', 't', 'e', 's', 't', '@', 'o', 'g', 'i', 'l', 'v', 'y', '.', 'c', 'o', 'm', '.', 'u', 'a', '\0'};

EmaiUniChar sContactFromName[]				= {'N', 'i', 'c', 'k', '\0'};
EmaiUniChar sContactFromAddress[]			= {'n', 'i', 'c', 'k', '@', 'm', 'y', 'd', 'o', 'm', 'a', 'i', 'n', '.', 'c', 'o', 'm', '\0'};

EmaiUniChar sMessageSubject[]				= {'N', 'e', 'w', ' ', 'b', 'o', 'o', 'k', '\0'};

/*Plain text of the message*/
EmaiConstCharPtr sPlainTextMessage			= "Hi Tom.\n\n"
												"I yesterday have bought the new book, hope, "
												"that it should be interesting.\n\n"
												"Good-bye!";
EmaiConstCharPtr sPlainTextMessage_			= "Hi Tom.\n\n"
												"I yesterday have bought the new book, "
												"hope, that it should be interesting. "
												"Look into the attached file.\n\n"
												"Good-bye!";
EmaiConstCharPtr sPlainTextMessage__		= "Hi Tom.\n\n"
												"I yesterday have bought the new book, "
												"hope, that it should be interesting. "
												"The book not badly looks, doesn't it?\n\n"
												"Look into the attached file.\n\n"
												"Good-bye!";

#define CONTENTID							"00b501c69b917b7e84500a00000a@developer"

/*Html text of the message*/
EmaiConstCharPtr sHtmlMessage				= "<html><body>Hi Tom.<br><br>"
												"I yesterday have bought the <b>new book</b>, "
												"hope, that it should be interesting. "
												"Look into the attached file.<br><br>"
												"Good-bye!</body></html>";
EmaiConstCharPtr sHtmlMessage_				= "<html><body>Hi Tom.<br><br>"
												"I yesterday have bought the <b>new book</b>, "
												"hope, that it should be interesting. "
												"The book not badly looks, doesn't it?"
												"<img src = \"cid:" CONTENTID "\"><br><br>"
												"Look into the attached file.<br><br>"
												"Good-bye!</body></html>";

/*Attachment file*/
EmaiUniChar sAttachmentFileName[]			= {'N', 'e', 'w', ' ', 'b', 'o', 'o', 'k', '.', 'z', 'i', 'p', '\0'};
EmaiUniChar sAttachmentFileFullPath[]		= {'C', ':', '\\', 'N', 'e', 'w', ' ', 'b', 'o', 'o', 'k', '.', 'z', 'i', 'p', '\0'};

/*Image file*/
EmaiUniChar sImageFileName[]				= {'N', 'e', 'w', ' ', 'b', 'o', 'o', 'k', '.', 'j', 'p', 'g', '\0'};
EmaiUniChar sImageFileFullPath[]			= {'C', ':', '\\', 'N', 'e', 'w', ' ', 'b', 'o', 'o', 'k', '.', 'j', 'p', 'g', '\0'};

static EmaiConstUniCharPtr
GetImageID()
{
	static EmaiUniChar sImageID[sizeof(CONTENTID)] = {0};
	if (sImageID[0] == 0)
	{
		for (size_t i = 0; i < sizeof(CONTENTID); ++ i)
			sImageID[i] = CONTENTID[i];
	}
	return sImageID;
}

int
main()
{
	/*Incorrect use of methods or impossibility to connect to a server methods raise*/
	/*an exception. Try/catch operators are used to handle the errors.*/
	
	try
	{
		/*Create the session*/
		SmtpCallback statusCallback;
		Emai::SmtpSession smtpSession(statusCallback, sTimeout);

		/*Connect to the server*/
		smtpSession.Connect(sMailServer);
		
		/*Authenticate the session*/
		smtpSession.Authenticate(sMailServerAccount, sMailServerPassword, sAuthType);

		/*Create the message from different parts*/
		MessageComposer messageComposer;
		messageComposer.SetText(sPlainTextMessage, EmaiUSASCII);
		messageComposer.SetHtml(sHtmlMessage_);
		messageComposer.AddInlineFromFile(sImageFileName, sImageFileFullPath, GetImageID());
		messageComposer.AddAttachmentFromFile(sAttachmentFileName, sAttachmentFileFullPath);

		/*Add contact fields to the message*/
		Emai::ContactList contactTo;
		contactTo.AddContact(sContactToName, sContactToAddress);
		Emai::ContactList contactFrom;
		contactFrom.AddContact(sContactFromName, sContactFromAddress);

		Emai::Message message = messageComposer.ComposeMessage(contactFrom, NULL, contactTo,
								NULL, NULL, sMessageSubject);

		/*Check if message is created successufully*/
		if (message != NULL)
		{
			/*Send the message*/
			smtpSession.SendMessage(message);
			
			/*Show the success notification*/
			std::cout << "Your message successfully has been sent." << std::endl << std::endl;
		}
		else
		{
			std::cout << "Your message hasn't been sent. There is a logical error in initialization object of MessageComposer class." << std::endl << std::endl;
		}
		
		/*Disconnect from the server*/
		smtpSession.Disconnect();
	}
	catch (Emai::Exception& exception)
	{
		/*Show the error code*/
		std::cout << "Your message hasn't been sent. Error # " << exception.GetErrorCode() << "." << std::endl << std::endl;
	}

	std::cout << "Done! Press <enter>." << std::endl;
	std::cin.get();

	return 0;
}
